pr_template = """#### Assignee*
- <작성자 이름>

#### JIRA Ticket*
- [JIRA Ticket](https://wantedlab.atlassian.net/browse/<JIRA ID>)

#### PR Summary*
- <여기에 summary를 작성>

#### PR Detail Descriptions

#### Screenshots / GIFs / Videos
- **AS-IS**
- **TO-BE**

#### Additional Comments (deployment server, reference, anything for reviewers..)"""
